<?php

namespace Ymina\CancelPending\Controller\Test;

use Magento\Sales\Model\Order;
use Ymina\CancelPending\Helper\Data;
use Transbank\Webpay\Model\WebpayOrderData;

class Index
{
    /**
     * @var Order
     */
    protected $_orderModel;

    /**
     * @var Data
     */
    protected $data;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    protected $localeDate;

    /**
     * @param Order $orderModel
     * @param Data $data
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate
     * @param \Transbank\Webpay\Model\WebpayOrderDataFactory $webpayOrderDataFactory
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        Order                                                $orderModel,
        Data                                                 $data,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Transbank\Webpay\Model\WebpayOrderDataFactory       $webpayOrderDataFactory,
        \Psr\Log\LoggerInterface                             $logger
    )
    {
        $this->_orderModel = $orderModel;
        $this->localeDate = $localeDate;
        $this->data = $data;
        $this->webpayOrderDataFactory = $webpayOrderDataFactory;
        $this->logger = $logger;


    }

    /**
     * Execute the cron
     *
     * @return void
     */
    public function execute()
    {
        $this->logger->info("ejecutando");
        $isEnabled = $this->data->getGeneralConfig(Data::ENABLE);
        $from_date = $this->data->getGeneralConfig(Data::FROM_DATE_TEXT);
        $this->logger->info("time->> " . $from_date);
        $todayStartOfDayDate = $this->localeDate->date(strtotime($from_date))->setTime(00, 00, 00)->format('Y-m-d H:i:s');
        $this->logger->info("ejecutando");

        if ($isEnabled) {

            $collection = $this->_orderModel->getCollection();
            $status = $this->data->getGeneralConfig(Data::PRIMARY_STATUS);
            $statusExp = explode(',', $status);
            if ($statusExp) {

                $collection->addFieldToFilter(
                    'status',
                    ['in' => $statusExp]
                )->addFieldToFilter('created_at', ["gt" => $todayStartOfDayDate]);

                $limitCollection = $this->data->getGeneralConfig(Data::BATCH);
                $collection->getSelect()->limit($limitCollection);

                foreach ($collection as $k => $order) {
                    $fechaEmitida = $order->getUpdatedAt();
                    $minutes = $this->data->getGeneralConfig(Data::MINUTES);
                    $fechaCaducidad = strtotime("+" . $minutes . " Minutes", strtotime($fechaEmitida));
                    $fechaAhora = strtotime(date('Y-m-d H:i:s'));
                    $pago_ok = $this->checkPay($order);
                    if (!$pago_ok) {
                        //echo $order->getIncrementId().'<br>';
                        if ($fechaAhora > $fechaCaducidad) {
                            try {

                                $order->cancel();
                                $order->setState('canceled')->setStatus(
                                    $this->data->getGeneralConfig(Data::SECONDARY_STATUS)
                                );
                                $order->addStatusToHistory($order->getStatus(), 'Cron: Orden cancelada por inactividad');

                            } catch (\Throwable $th) {
                            }
                        }
                    } else {
                        $order->addStatusToHistory($order->getStatus(), 'Cron: Orden no cancelada, Tiene un pago ok Asociado');
                    }
                    $order->save();
                }
            }
            return;
        }
        return;
    }


    /**
     * @param $order
     * @return bool
     */
    protected function checkPay($order)
    {
        $pay = false;
        try {
            $webpayOrderDataModel = $this->webpayOrderDataFactory->create();
            $webpayOrderData = $webpayOrderDataModel->load($order->getIncrementId(), 'order_id');
            $paymentStatus = $webpayOrderData->getPaymentStatus();
            if ($paymentStatus == WebpayOrderData::PAYMENT_STATUS_SUCCESS) {
                $pay = true;
            }
        } catch (\Exception $e) {
            return false;
        }
        return $pay;
    }
}
