<?php

namespace Ymina\Checkout\Model;

use Magento\Checkout\Model\ConfigProviderInterface;
use Magento\Theme\ViewModel\Block\Html\Header\LogoPathResolver;
use Magento\Theme\ViewModel\Block\Html\Header\LogoSizeResolver;
use Magento\Store\Model\ScopeInterface;

class AdditionalConfigVars extends \Magento\Theme\Block\Html\Header\Logo implements ConfigProviderInterface
{
    const XML_PATH_HEADER_LOGO = 'design/header/logo_src';
    const XML_PATH_SHOW_RECEIPT_TYPE = 'checkout/options/show_receipt_type';

    /**
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\MediaStorage\Helper\File\Storage\Database $fileStorageHelper
     * @param LogoPathResolver $logoPathResolver
     * @param LogoSizeResolver $logoSizeResolver
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context   $context,
        \Magento\MediaStorage\Helper\File\Storage\Database $fileStorageHelper,
        protected LogoPathResolver                         $logoPathResolver,
        protected LogoSizeResolver                         $logoSizeResolver,
        array                                              $data = [])
    {
        parent::__construct($context, $fileStorageHelper, $data);
    }


    /**
     * @return array
     */
    public function getConfig()
    {
        $storeName = $this->getThemeName() ? $this->getThemeName() : $this->getLogoAlt();


        $logoWidth = $this->logoSizeResolver->getWidth()
            ? $this->logoSizeResolver->getWidth()
            : $this->getLogoWidth();

        $logoHeight = $this->logoSizeResolver->getHeight()
            ? $this->logoSizeResolver->getHeight()
            : $this->getLogoHeight();

        $logo = $this->getLogo();
        $logoSrc = $logo ?: $this->getLogoSrc();

        $additionalConfig['logoConfig'] = [
            'src' => $logoSrc,
            'alt' => $this->getLogoAlt(),
            'width' => $logoWidth,
            'height' => $logoHeight,
            'title' => $this->escapeHtmlAttr($storeName),
            'logo' => $this->getLogo()
        ];
        $additionalConfig['show_receipt_type'] = (bool)$this->getShowReceiptType();
        return $additionalConfig;
    }


    /**
     * @return string|void
     */
    private function getLogo()
    {
        $folderName = \Magento\Config\Model\Config\Backend\Image\Logo::UPLOAD_DIR;
        $storeLogoPath = $this->_scopeConfig->getValue(self::XML_PATH_HEADER_LOGO, ScopeInterface::SCOPE_STORE);
        if (!$this->_scopeConfig->getValue(self::XML_PATH_HEADER_LOGO, ScopeInterface::SCOPE_STORE)) {
            return;
        }
        return $this->getUrl() . 'media/' . $folderName . '/' . $storeLogoPath;
    }

    private function getShowReceiptType()
    {
        return (int)$this->_scopeConfig->getValue(self::XML_PATH_SHOW_RECEIPT_TYPE, ScopeInterface::SCOPE_STORE);
    }
}