<?php

namespace Ymina\Checkout\Plugin;

use Magento\Checkout\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException as NoSuchEntityExceptionAlias;
use Magento\Quote\Api\Data\AddressInterface;
use Magento\Quote\Model\GuestCart\GuestShippingMethodManagement;
use Magento\Store\Model\StoreManager;
use Ymina\Checkout\Helper\Data;

class EstimateShipping
{

    /**
     * @var \Magento\Framework\App\Request\DataPersistor
     */
    private $_dataPersistor;


    private $_logger;
    private Session $checkoutSession;
    private Data $helper;
    private StoreManager $storeManager;
    private ScopeConfigInterface $scopeConfig;

    public function __construct(
        \Magento\Framework\App\Request\DataPersistor $dataPersistor,
        \Psr\Log\LoggerInterface                     $logger,
        StoreManager                                 $storeManager,
        ScopeConfigInterface                         $scopeConfig,
        Data                                         $helper)
    {
        $this->_dataPersistor = $dataPersistor;
        $this->_logger = $logger;
        $this->helper = $helper;
        $this->storeManager = $storeManager;
        $this->scopeConfig = $scopeConfig;
    }


    /**
     * @param GuestShippingMethodManagement $subject
     * @param $result
     * @param mixed $cartId
     * @param AddressInterface $address
     */
    public function afterEstimateByExtendedAddress(
        GuestShippingMethodManagement $subject,
                                      $result, $cartId, AddressInterface $address)
    {
        $typeDelivery = $this->_dataPersistor->get('typeDelivery');

        $methods = [];
        if ($this->getCountry() !== "PE") {
            foreach ($result as $method) {
                if ($method->getCarrierCode() == 'storepickup' && $typeDelivery == 'pickupInStore') {
                    $methods[] = $method;
                } else {
                    $methods = [$method];
                }
            }

        } else {
            foreach ($result as $method) {
                if ($method->getCarrierCode() == 'perustorepickup' && $typeDelivery == 'pickupInStore') {
                    $methods[] = $method;
                } else if ($method->getCarrierCode() != 'perustorepickup' && $typeDelivery != 'pickupInStore') {
                    $methods = [$method];
                }
            }
        }
        return $methods;

    }

    private function getCountry()
    {
        return $this->scopeConfig->getValue('general/country/default',
            \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE);
    }


}
