define([
    'jquery',
    'underscore',
    'uiRegistry',
    'Magento_Ui/js/form/element/select',
    'mage/translate',
    'mage/url',
    'Ymina_Checkout/js/model/shipping-type',
    'Magento_Checkout/js/model/shipping-rate-processor/new-address',
    'Magento_Checkout/js/model/quote',
], function ($, _, registry, Select, $t, $url, sippingType, newAddress, quote) {

    return Select.extend({
        defaults: {
            skipValidation: false,
            imports: {
                update: '${ $.parentName }.region_id:value',
                city: '${ $.parentName }.city'
            },
            options: [{title: $t("Comuna*"), value: "", label: $t("Comuna*")}],
            visible: false
        },

        /**
         * {@inheritdoc}
         */
        initialize: function () {
            this._super();
            if (quote.shippingAddress() && quote.shippingAddress().city) {
                this.value(quote.shippingAddress().city)
            }
            return this;
        },

        /**
         * On region update we check for city
         *
         * @param {string} regionId
         */
        update: function (regionId) {
            self = this;
            let _url = $url.build("comuna");
            const data = {
                regionId: regionId
            }

            $.ajax({
                url: _url,
                data: data,
                beforeSend: function () {
                    $('body').trigger('processStart');
                },
                success: function (data) {
                    try {
                        if (typeof data === "object" && 'options' in data && data.options.length) {
                            let options = [];
                            options = data.options;
                            self.options(options);
                        } else if (data !== "object" && "result" in data && data.result) {
                            let options = data.result.map(value => {
                                return {
                                    title: value.nombre,
                                    value: value.nombre,
                                    label: value.nombre
                                }
                            })
                            self.options(options);
                        } else {
                            const objectEm = []
                            self.options(objectEm);
                        }

                        if (quote.shippingAddress().city) {
                            self.value(quote.shippingAddress().city)
                        }


                    } catch (e) {
                        console.log(e)
                    }
                },
                complete: function () {
                    $('body').trigger('processStop');
                }
            })
        },

        hasChanged: function () {

            if(this.value() && this.value() !== "") {
                quote.shippingAddress().city = this.value();
                newAddress.getRates(quote.shippingAddress())
            }
        }
    })
})
