<?php

namespace Ymina\CustomSearch\Block;

use Magento\Eav\Model\AttributeRepository;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\View\Element\Template;
use Ymina\CustomSearch\Model\ConfigProvider;
use Ymina\CustomSearch\Model\CustomSearchIndex;

class Search extends \Magento\Framework\View\Element\Template
{
    const ATTRIBUTE_PRODUCT_TYPE = "catalog_product";
    const FILTER_TYPE_N = "neumaticos";
    const FILTER_TYPE_L = "llantas";
    const FIELD_ALLOW = ['tipo_de_vehiculo', 'ancho_offset', 'ancho_neumatico', 'apernadura', 'aro', 'perfil_neumatico'];
    private AttributeRepository $attributeRepository;
    private Json $json;
    private SearchCriteriaBuilder $searchCriteriaBuilder;
    private ConfigProvider $configProvider;
    private CustomSearchIndex $customSearchIndex;

    public function __construct(Json $json, AttributeRepository $attributeRepository, CustomSearchIndex $customSearchIndex, SearchCriteriaBuilder $searchCriteriaBuilder, ConfigProvider $configProvider, Template\Context $context, array $data = [])
    {
        parent::__construct($context, $data);
        $this->attributeRepository = $attributeRepository;
        $this->json = $json;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->configProvider = $configProvider;
        $this->customSearchIndex = $customSearchIndex;
    }

    public function getConfig(): bool|array|string
    {
        $config = $this->getAttributes();
        return $this->json->serialize($config);
    }

    protected function getAttributes(): array
    {
        $filterType = [];
        $filterType["neumaticos"] = $this->getFilters(self::FILTER_TYPE_N);
        $filterType["llantas"] = $this->getFilters(self::FILTER_TYPE_L);
        return $filterType;
    }

    private function getFilters($type): array
    {
        $filter = [];
        if ($type === self::FILTER_TYPE_L) {
            $attributesCode = $this->configProvider->getLLantas();
        } else {
            $attributesCode = $this->configProvider->getPneumatics();
        }
        $criteria = $this->searchCriteriaBuilder->addFilter("attribute_code", ["in" =>$attributesCode], "in")->create();
        $attributes = $this->attributeRepository->getList(self::ATTRIBUTE_PRODUCT_TYPE, $criteria);
        foreach ($attributes->getItems() as $attribute) {
            $filter[] = [
                "name" => $attribute->getFrontendLabel(),
                "code" => $attribute->getAttributeCode(),
                "input_type" => $attribute->getFrontendInput(),
                "options" => $this->getOptions($attribute)
            ];
        }
        return $filter;
    }

    private function getOptions(\Magento\Eav\Api\Data\AttributeInterface $attribute): array
    {
        $options = [];
        foreach ($attribute->getOptions() as $option) {
            $valueDb = $this->customSearchIndex->findAttributeValue($option->getValue(), $attribute->getAttributeCode());
            if ($valueDb) {
                $options[] = [
                    "label" => $option->getLabel(),
                    "value" => $option->getValue()
                ];
            }
        }
        return $options;
    }
}
