<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Ymina\HomeDelivery\Model\Carrier;

use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Shipping\Model\Rate\Result;
use Ymina\HomeDelivery\Api\Data\FeeInterface;
use Ymina\HomeDelivery\Model\FeeRepository;

class HomeDelivery extends \Magento\Shipping\Model\Carrier\AbstractCarrier implements
    \Magento\Shipping\Model\Carrier\CarrierInterface
{
    /**
     * @var string
     */
    protected $_code = 'homedelivery';

    /**
     * @var bool
     */
    protected $_isFixed = true;

    /**
     * @var \Magento\Shipping\Model\Rate\ResultFactory
     */
    protected $_rateResultFactory;

    /**
     * @var \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory
     */
    protected $_rateMethodFactory;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    protected $_searchCriteriaBuilder;

    /**
     * @var FeeRepository
     */
    protected $_feeRepository;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    const COMMUNES_NOT_AVAILABLE = [ ];

    /**
     * Constructor
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory
     * @param \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface          $scopeConfig,
        \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory  $rateErrorFactory,
        \Psr\Log\LoggerInterface                                    $logger,
        \Magento\Shipping\Model\Rate\ResultFactory                  $rateResultFactory,
        \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory,
        \Magento\Framework\Api\SearchCriteriaBuilder                $searchCriteriaBuilder,
        \Ymina\HomeDelivery\Model\FeeRepository                     $feeRepository,
        array                                                       $data = []
    )
    {
        $this->_rateResultFactory = $rateResultFactory;
        $this->_rateMethodFactory = $rateMethodFactory;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->_feeRepository = $feeRepository;
        $this->logger = $logger;
        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);
    }

    /**
     * {@inheritdoc}
     */
    public function collectRates(RateRequest $request)
    {
        if (!$this->getConfigFlag('active')) {
            return false;
        }

        $shippingPrice = $this->getConfigData('price');
        $result = $this->_rateResultFactory->create();
        $packageWeight = $request->getPackageWeight();

        if (in_array($request->getDestCity(), self::COMMUNES_NOT_AVAILABLE)) {
            return false;
        }

        $continue = false;

        foreach ($request->getAllItems() as $item) {
            $categoryIds = $item->getProduct()->getCategoryIds();
            if (in_array("219", $categoryIds) || in_array("27", $categoryIds)) {
                $continue = true;
            }
        }

        if ($continue) {
            $this->_searchCriteriaBuilder->addFilter(FeeInterface::COMMUNE, $request->getDestCity());
            $searchCriteria = $this->_searchCriteriaBuilder->create();
            $fees = $this->_feeRepository->getList($searchCriteria);
            if ($fees->getTotalCount()) {
                foreach ($fees->getItems() as $item) {
                    if ($packageWeight <= 30) {
                        $shippingPrice = 30 * (float)$item->getCost();
                    }
                    if ($packageWeight > 30) {
                        $shippingPrice = $packageWeight * (float)$item->getCost();
                    }
                }
            }
        }

        if ($request->getDestRegionId() == "2041") {
            $shippingPrice = 0;
            $this->logger->info("Entro filtro region");
        }

        if ($shippingPrice !== false) {
            $method = $this->_rateMethodFactory->create();

            $method->setCarrier($this->_code);
            $method->setCarrierTitle($this->getConfigData('title'));

            $method->setMethod($this->_code);
            $method->setMethodTitle($this->getConfigData('name'));

            if ($request->getFreeShipping() === true) {
                $shippingPrice = '0.00';
            }

            $method->setPrice($shippingPrice);
            $method->setCost($shippingPrice);

            $result->append($method);
        }

        return $result;
    }

    /**
     * getAllowedMethods
     *
     * @return array
     */
    public function getAllowedMethods()
    {
        return [$this->_code => $this->getConfigData('name')];
    }
}
