<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Ymina\StorePickup\Plugin\Frontend\Amasty\StorePickupWithLocator;

use Amasty\StorePickupWithLocator\Model\ConfigProvider;
use Amasty\StoreLocator\Model\LocationFactory;


class CustomerData
{
    private $locationFactory;
    private $logger;

    /**
     * @var ConfigProvider
     */
    private $configProvider;

    /**
     * @param LocationFactory $locationFactory
     * @param ConfigProvider $configProvider
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        LocationFactory          $locationFactory,
        ConfigProvider           $configProvider,
        \Psr\Log\LoggerInterface $logger)
    {
        $this->locationFactory = $locationFactory;
        $this->logger = $logger;
        $this->configProvider = $configProvider;

    }

    public function afterGetSectionData(
        \Amasty\StorePickupWithLocator\CustomerData\LocationData $subject,
                                                                 $result
    )
    {
        if ($this->isStorePickupEnabled()) {
            try {

                $stores = [];

                foreach ($result['stores'] as $key => $item) {
                    /**
                     * @var \Amasty\Storelocator\Model\Location $location
                     */
                    $location = $this->locationFactory->create()->load($item['id']);
                    if ($location && $location->getAttributes()) {
                        $attributes = $location->getAttributes();
                        if (isset($attributes["sucursal"]) && isset($attributes["sucursal"]["option_title"])) {
                            if ($attributes["sucursal"]["option_title"][0] == "Santiago" || $attributes["sucursal"]["option_title"][0] == "Regiones") {
                                $stores[] = $item;
                            }
                        }
                    }
                }
                $result['stores'] = $stores;
                $this->logger->info("result..>" . json_encode(count($stores)));
                return $result;
            } catch (\Exception $e) {
                $this->logger->info($e->getMessage());
                return $result;
            }
        }
    }

    private function isStorePickupEnabled()
    {
        return $this->configProvider->isStorePickupEnabled();
    }
}

